%%
%
%
%
ccc 
load('attendee_data');

%% EXTRACT FEATURE FROM THE DATA
%
% WE WILL USE INTEREST + MOTIVS + JOBS + FAMILY NAMES
% 
features = [d.interests d.motivs d.jobs_onehot d.names_coding];

%% 1ST LAYER RBM
maxepoch   = 1E3; % THE BIGGER THE BETTER
numHid     = 1E3;       
CD         = 1;         
discretize = 0;       
numCases   = 1;      
lr         = 1E-2;
[visHid, visBiases, hidBiases, learning_curve] ...
    = train_rbm(features, maxepoch, numHid, CD, discretize, numCases, lr);
l1.visHis         = visHid;
l1.visBiases      = visBiases;
l1.hidBiases      = hidBiases;
l1.learning_curve = learning_curve;
l1.out            = encode_rbm( features, visHid, visBiases, hidBiases, discretize, CD);
save('l1.mat', 'l1'); 

%% 2ND LAYER RBM
ccc;
load('l1.mat'); 
maxepoch   = 1E3; % THE BIGGER THE BETTER
numHid     = 1E2;       
CD         = 1;         
discretize = 0;       
numCases   = 1;      
lr         = 1E-2;
[visHid, visBiases, hidBiases, learning_curve] ...
    = train_rbm(l1.out, maxepoch, numHid, CD, discretize, numCases, lr);
l2.visHis         = visHid; 
l2.visBiases      = visBiases; 
l2.hidBiases      = hidBiases; 
l2.learning_curve = learning_curve;
l2.out            = encode_rbm( l1.out, visHid, visBiases, hidBiases, discretize, CD);
save('l2.mat', 'l2');

%% 3RD LAYER RBM
load('l2.mat'); 
maxepoch   = 1E3;      
numHid     = 1E2;       
CD         = 1;         
discretize = 0;       
numCases   = 1;      
lr         = 1E-2; 
[visHid, visBiases, hidBiases, learning_curve] ...
    = train_rbm(l2.out, maxepoch, numHid, CD, discretize, numCases, lr);
l3.visHis         = visHid; 
l3.visBiases      = visBiases; 
l3.hidBiases      = hidBiases; 
l3.learning_curve = learning_curve;
l3.out            = encode_rbm( l2.out, visHid, visBiases, hidBiases, discretize, CD);
save('l3.mat', 'l3');

%% t-SNE
ccc;
addpath(genpath('drtoolbox')); % <= MEX COMPILE MIGHT BE REQUIRED. READ 'README.TXT' INSIDE. 
load('l3');
no_dims = 2;
[tsne_out] = compute_mapping(l3.out, 'tSNE', no_dims);
save('tsne_out.mat', 'tsne_out');

%% DO SPECTRAL CLUSTERING 
ccc;
rng(11);
load('l2');
load('tsne_out');
sigma = 10;
A = get_affinitymtx(tsne_out, sigma);
D = diag(sum(A));
L = D^(-0.5)*A*D^(-0.5);
[e, v] = eig(L);
k = 4; 
X = e(:, 1:k);
n = size(A, 1);
Y = zeros(n, k);
for i = 1:n
    Y(i, :) = X(i, :) / norm(X(i, :));
end
nr_label = 16; % NUMBER OF GROUPS 
% DO KMEAN JUST FOR COMPUTING THE CENTER 
[~, centers] = kmeans(Y, nr_label, 'Distance', 'sqeuclidean');

% DISTANCE BASED CLUSTERING WITH SIMILAR GROUP CARDINALITY CONSTRAINT
label_randidx = randperm(nr_label);
labels = zeros(n, 1);
indexset = 1:n;
nn = 0; flag = 1;
while flag
    for i = label_randidx
        % FOR EACH CLASS, SELECT ONE DATA 
        curr_center = centers(i, :);
        dists = zeros(length(indexset), 1);
        for j = 1:length(indexset)
            idx = indexset(j);
            curr_data = Y(idx, :);
            curr_dist = norm(curr_data-curr_center);
            dists(j) = curr_dist;
        end
        [~, temp_idx] = min(dists); % DATA INDEX
        min_idx = indexset(temp_idx);
        labels(min_idx) = i;
        indexset(temp_idx) = [];
        nn = nn + 1;
        if nn > n
            flag = 0;
            break;
        end
    end
end
% SAVE 
save('labels.mat', 'labels', 'nr_label');

% PLOT CLUSTERING RESULTS 
if 1
    fig_cluster = figure(); clf; hold on;
    colors = jet(nr_label);
    for i = 1:length(tsne_out)
        for j = i:length(tsne_out)
            ti = tsne_out(i, :);
            tj = tsne_out(j, :);
            dij = norm(ti - tj);
            if dij < 3
                plot([ti(1) tj(1)], [ti(2) tj(2)], 'k-', 'LineWidth', 2);
            end
        end
    end
    for i = 1:nr_label
        idx = find(labels == i);
        % fprintf('[%d/%d] n: %d\n', i, nr_label, length(idx));
        plot(tsne_out(idx, 1), tsne_out(idx, 2), 'o', 'Color', 'k' ...
            , 'LineWidth', 2, 'MarkerSize', 12, 'MarkerFaceColor', colors(i, :));
    end 
    grid on;
    axis equal;
    title(sprintf('Clustering Result (%d Groups)', nr_label));
    set(fig_cluster,'PaperPositionMode','auto')
    print (fig_cluster , '-dpng', 'fig_cluster.png') ;
end

%% GROUPING BASED ON SPECTRAL CLUSTERING (NOT USED)
ccc
load('attendee_data');
load('labels.mat');
fileID = fopen('groups.txt','w');
for i = 1:nr_label
    idx = find(labels == i);
    nr_idx = length(idx);
    fprintf(fileID, '\n\n--------------------------------------------------------');
    fprintf(fileID, 'Group %d -------------------------------------------------------- \n', i);
    for j = 1:nr_idx
        ii = idx(j);
        curr_name = d.names{ii};
        curr_job  = d.jobs_raw{ii};
        curr_aff  = d.affiliations{ii};
        curr_int  = d.interests_raw{ii};
        curr_mot  = d.motivs_raw{ii};
        
        curr_int2 = strrep(curr_int, sprintf('\n'), '/');
        curr_mot2 = strrep(curr_mot, sprintf('\n'), '/');
        fprintf(fileID, '[Group %d] [%d/%d] ̸: [%s] : [%s] Ҽ: [%s] ɺо: [%s] : [%s]\n' ...
            , i, j, nr_idx, curr_name, curr_job, curr_aff, curr_int2, curr_mot2);
    end 
end
fclose(fileID);

%% FINAL GROUPING WITH 120 ATTENDEES + 50> ADMINS 
ccc
rng(16);
load('attendee_data');
load('labels.mat');
load('admins.mat');

% RANDOM ADMIN GROUPING 
admin_group = cell(nr_label, 1);
rand_idx = randperm(admins.nr_people);
for i = 1:admins.nr_people
    ii = rand_idx(i);
    mod_idx = mod(i, nr_label)+1;
    admin_group{mod_idx} = [admin_group{mod_idx} ; ii];
end

fileID = fopen('total_groups.txt','w');
for i = 1:nr_label
    idx = find(labels == i);
    nr_idx = length(idx);
    fprintf(fileID, '\r\n\r\n--------------------------------------------------------');
    fprintf(fileID, 'Group %d -------------------------------------------------------- \r\n', i);
    
    curr_admingroup = admin_group{i};
    nr_idx2 = length(curr_admingroup);
    for j = 1:nr_idx2
        ii = curr_admingroup(j);
        curr_name = admins.names{ii};
        curr_affl = admins.affli{ii};
        % fprintf('%d - %s / %s \n', i, curr_name, curr_affl);
        fprintf(fileID, '[Group %d] [%d/%d] ̸: [%s] : [%s] \r\n', i, j, nr_idx+nr_idx2, curr_name, curr_affl);
    end
    for j = 1:nr_idx
        ii = idx(j);
        curr_name = d.names{ii};
        curr_job  = d.jobs_raw{ii};
        curr_aff  = d.affiliations{ii};
        curr_int  = d.interests_raw{ii};
        curr_mot  = d.motivs_raw{ii};
        curr_int2 = strrep(curr_int, sprintf('\n'), '/'); 
        curr_mot2 = strrep(curr_mot, sprintf('\n'), '/');
        fprintf(fileID, '[Group %d] [%d/%d] ̸: [%s] : [%s] Ҽ: [%s] ɺо: [%s] : [%s] \r\n' ...
            , i, j+nr_idx2, nr_idx+nr_idx2, curr_name, curr_job, curr_aff, curr_int2, curr_mot2);
    end 
end
fclose(fileID);

%%
















